/*
This is a program I've written to prototype background collision detection and moving platforms for my NES platformer.

Currently, moving platforms aren't completely solid, they can be stood on, but moved through from below (think Item-1, 2, 3 in Mega Man 2).
Also, behavior for being pushed into a wall vertically by a platform isn't yet defined.

main.c controls the sprites and does all the drawing. The other files comprise the engine and don't contain any Allegro-dependant code.

Tested with Allegro 4.2.3.

Special thanks to tokumaru whose method of platform/player movement I adapted and used (http://nesdev.parodius.com/bbs/viewtopic.php?t=6849).

http://morphcat.de
*/

#include <allegro.h>
#include <math.h>
#include "map.h"
#include "sprite.h"

#define CANVASW 256
#define CANVASH 224
#define SCREENW 512
#define SCREENH 448




struct {
	BITMAP *bkbuf;
	int fps;
	int fps_count;
	int math_pending;
	int frame_count;
} g;



void render()
{
	clear(g.bkbuf);
	
	int x,y;
	int i;
	for(i=0;i<MAX_SPR;i++) {
		if(spr[i].a) {
			x = spr[i].x;
			y = spr[i].y;
			int x2 = spr[i].x+spr[i].w;
			int y2 = spr[i].y+spr[i].h;
			if(i==0)
				rect(g.bkbuf, x, y, x2, y2, (spr[i].collision_flag) ? makecol(240,140,140) : makecol(160,160,160) );
			else
				rect(g.bkbuf, x, y, x2, y2, (spr[i].collision_flag) ? makecol(200,50,0) : makecol(50,100,200) );
			if(spr[i].ground_flag)
				line(g.bkbuf, x, y2, x2, y2, makecol(255,255,255));
		}
	}
	
	for(x=0;x<16;x++) {
		for(y=0;y<15;y++) {
			if(map_get_tile(x,y)==1)
				rectfill(g.bkbuf,x*16,y*16,x*16+15,y*16+15, ((x+y)&1) ? makecol(128,100,60) : makecol(108,80,40) );
		}
	}
	
	textprintf_ex(g.bkbuf, font, 0, 0, makecol(255,255,0), 0, "%d fps", g.fps);
	textprintf_ex(g.bkbuf, font, 0, CANVASH-8, makecol(255,255,0), 0, "Arrow Keys: Move");
	
	vsync();
	acquire_screen();
	stretch_blit(g.bkbuf, screen, 0, 0, CANVASW, CANVASH, 0, 0, SCREENW, SCREENH);
	release_screen();
}


void do_math()
{
	//control player
	if(key[KEY_UP]) {
		spr[0].vy -= 0.5f;
		spr[0].platform = -1; //detach player from platform
	}
	else if(key[KEY_DOWN])
		spr[0].vy += 0.5f;
		
	if(key[KEY_LEFT])
		spr[0].vx -= 0.5f;
	else if(key[KEY_RIGHT])
		spr[0].vx += 0.5f;
		
	if(mouse_b&1) {
		spr[0].x = mouse_x / 2;
		spr[0].y = mouse_y / 2;
	}
	
	//control platform 1
	if(g.frame_count%300<150)
		spr[1].vx = 1.0f;
	else
		spr[1].vx = -1.0f;
		
	if(mouse_b&2) {
		spr[1].x = (float)mouse_x / ((float)SCREENW/CANVASW);
		spr[1].y = (float)mouse_y / ((float)SCREENH/CANVASH);
	}
		
	//control platform 2
	if(g.frame_count%200<100)
		spr[2].vy = 0.65f;
	else
		spr[2].vy = -0.65f;
		
	//control platform 3
	spr[3].vx = sin(g.frame_count/40.0f);
	spr[3].vy = cos(g.frame_count/40.0f);
	
	sprites_update();
	g.frame_count++;
}



void do_fps()
{
	g.fps = g.fps_count;
	g.fps_count = 0;
}
END_OF_FUNCTION(do_fps)

void timer60bps()
{
	g.math_pending++; //do_math 60 times per second
}
END_OF_FUNCTION(timer60bps)


int main(int argc, char **argv)
{
	int depth;
    allegro_init();
	if ((depth = desktop_color_depth()) != 0) {
		set_color_depth(depth);
	}
	else
		set_color_depth(32);
#ifdef FULLSCREEN
	set_gfx_mode(GFX_AUTODETECT_FULLSCREEN, SCREENW, SCREENH, 0, 0);
#else
    set_gfx_mode(GFX_AUTODETECT_WINDOWED, SCREENW, SCREENH, 0, 0);
#endif
	g.bkbuf = create_bitmap(CANVASW,CANVASH);
	//set_display_switch_mode(SWITCH_BACKGROUND);
    install_keyboard();
	install_mouse();
	install_timer();
	show_os_cursor(0); //MOUSE_CURSOR_ARROW
	show_mouse(screen);
	
	LOCK_VARIABLE(g.fps_count);
	LOCK_VARIABLE(g.fps);
	LOCK_FUNCTION(do_fps);
	LOCK_VARIABLE(g.math_pending);
	LOCK_FUNCTION(timer60bps);
	g.fps_count = 0;
	g.fps = 0;
	install_int_ex(do_fps, MSEC_TO_TIMER(1000));
	g.math_pending = 0;
	install_int_ex(timer60bps, BPS_TO_TIMER(60));
	
	sprites_init();
	
	int i;
	while(!key[KEY_ESC]) {
		for(i=0;i<g.math_pending;i++)
			do_math();
		g.math_pending=0;
		render();
		g.fps_count++;
		//rest(10);
	}
    
	remove_int(timer60bps);
	remove_int(do_fps);
	remove_timer();
	destroy_bitmap(g.bkbuf);
	allegro_exit();
	return 0;
}
END_OF_MAIN()


